==> down.php?SDK=cocos_V1.20.3.2.zip: cocos_V1.20.3.2/Android/sample/Classes/IAPV4SubscriptionProductInfoView.cpp <==
/**
 * IAPV4SubscriptionProductInfoView.cpp
 * HIVE_SDK_Cocos_Sample
 * 
 * Created by disker on 2020. 05. 27.
 * Copyright © 2020년 GCP. All rights reserved.
*/

#include <string>
#include "IAPV4SubscriptionProductInfoView.hpp"
#include "ResultUtils.hpp"
#include "IAPV4Test.hpp"
#include "cocos2d.h"

using namespace hive;

bool IAPV4SubscriptionProductInfoView::init()
{
    if (!BaseView::init())
        return false;

    return true;
}

IAPV4SubscriptionProductInfoView *IAPV4SubscriptionProductInfoView::create(std::vector<std::string> const pidList)
{
    IAPV4SubscriptionProductInfoView *ret = new (std::nothrow) IAPV4SubscriptionProductInfoView();
    if (ret && ret->initialize(pidList))
    {
        ret->autorelease();
        return ret;
    }
    else
    {
        delete ret;
        ret = nullptr;
        return nullptr;
    }
}

bool IAPV4SubscriptionProductInfoView::initialize(std::vector<std::string> const pidList)
{
    if (!IAPV4SubscriptionProductInfoView::init())
        return false;

    this->pidList.clear();
    this->pidList.assign(pidList.begin(), pidList.end());

    setUseBackButton(true);
    initGroupPidList();

    addLabel("GROUP_1");
    for (auto item : pidList_group_1)
    {
        int index = getIndexFromVector(pidList, item);
        if (index != -1)
            addButton(item.c_str(), index);
    }

    addDottedLine();

    addLabel("GROUP_2");
    for (auto item : pidList_group_2)
    {
        int index = getIndexFromVector(pidList, item);
        if (index != -1)
            addButton(item.c_str(), index);
    }

    StartActivityIndicator();
    Postbox::posts([=](const Postbox::Posts *posts, Postbox::Error error) {
        if (posts == nullptr)
        {
            EndActivityIndicator();
            return;
        }

        Postbox::SubscriptionList subscriptionList = posts->subscription_list;
        if (subscriptionList.empty())
        {
            EndActivityIndicator();
            return;
        }

        for (auto subs : subscriptionList)
        {
            int index = getIndexFromVector(pidList, subs.product_id);
            if (index == -1)
                continue;

            cocos2d::ui::Button *button = (cocos2d::ui::Button *)BaseView::getUIWidget(index);
            button->setTitleColor(cocos2d::Color3B::RED);

            if (getIndexFromVector(pidList_group_1, subs.product_id) != -1)
            {
                group_1_item = subs.product_id;
            }
            else
            {
                if (getIndexFromVector(pidList_group_2, subs.product_id) != -1)
                {
                    group_2_item = subs.product_id;
                }
            }
        }
        EndActivityIndicator();
    });

    return true;
}

void IAPV4SubscriptionProductInfoView::initGroupPidList()
{
    pidList_group_1.clear();
    pidList_group_2.clear();

#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    pidList_group_1.push_back("sub01");
    pidList_group_1.push_back("sub10");
    pidList_group_1.push_back("sub11");
    pidList_group_1.push_back("sub30");

    pidList_group_2.push_back("sub013");
    pidList_group_2.push_back("sub103");
    pidList_group_2.push_back("sub303");
    pidList_group_2.push_back("sub113");
#else
    pidList_group_1.push_back("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.arshive003");
    pidList_group_1.push_back("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.arshive002");
    pidList_group_1.push_back("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.arshive001");

    pidList_group_2.push_back("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.arshive033");
    pidList_group_2.push_back("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.arshive032");
    pidList_group_2.push_back("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.arshive031");
#endif
}

void IAPV4SubscriptionProductInfoView::onButtonEvent(int buttonTag)
{
    std::string marketPid = pidList[buttonTag];
    std::string oldMarketPid = "";

    int index = getIndexFromVector(pidList_group_1, marketPid);
    if (index != -1)
    {
        oldMarketPid = group_1_item;
    }
    else
    {
        index = getIndexFromVector(pidList_group_2, marketPid);
        if (index != -1)
        {
            oldMarketPid = group_2_item;
        }
    }

    IAPV4::purchaseSubscriptionUpdate(marketPid, oldMarketPid, "", [=](ResultAPI const &result, IAPV4Receipt const &receipt) {
        ResultUtils::logHead("IAPV4", "purchaseSubscriptionUpdate", result);
        if (ResultUtils::errorCheckResultAPI(result))
        {
            StartActivityIndicator();

            receiptVerify(receipt, Postbox::subscription, [=]() {
                EndActivityIndicator();
            });
        }
    });
}

void IAPV4SubscriptionProductInfoView::receiptVerify(hive::IAPV4Receipt receipt, Postbox::ItemType itemType, std::function<void()> callback)
{
    Postbox::verifyReceipt(receipt.bypassInfo, itemType, [=](const Postbox::VerifyReceipt *data, Postbox::Error error) mutable {
        if (data != nullptr)
        {
            //영수증 검증 성공, 피니쉬 처리.
            IAPV4::transactionFinish(receipt.product.marketPid, [=](ResultAPI const &result, std::string marketPid) mutable {

                if (callback)
                    callback();
            });
        }
        else
        {
            if (callback)
                callback();
        }
    });
}

int IAPV4SubscriptionProductInfoView::getIndexFromVector(std::vector<std::string> v, std::string data)
{
    auto itr = std::find(v.begin(), v.end(), data);
    if (itr != v.end())
    {
        return itr - v.begin();
    }
    return -1;
}

==> down.php?SDK=cocos_V1.21.1.1.zip: cocos_V1.21.1.1/Android/sample/Classes/IAPV4SubscriptionProductInfoView.cpp <==
/**
 * IAPV4SubscriptionProductInfoView.cpp
 * HIVE_SDK_Cocos_Sample
 * 
 * Created by disker on 2020. 05. 27.
 * Copyright © 2020년 GCP. All rights reserved.
*/

#include <string>
#include "IAPV4SubscriptionProductInfoView.hpp"
#include "ResultUtils.hpp"
#include "IAPV4Test.hpp"
#include "cocos2d.h"

using namespace hive;

bool IAPV4SubscriptionProductInfoView::init()
{
    if (!BaseView::init())
        return false;

    return true;
}

IAPV4SubscriptionProductInfoView *IAPV4SubscriptionProductInfoView::create(std::vector<std::string> const pidList, std::string const &iapPayload)
{
    IAPV4SubscriptionProductInfoView *ret = new (std::nothrow) IAPV4SubscriptionProductInfoView();
    if (ret && ret->initialize(pidList, iapPayload))
    {
        ret->autorelease();
        return ret;
    }
    else
    {
        delete ret;
        ret = nullptr;
        return nullptr;
    }
}

bool IAPV4SubscriptionProductInfoView::initialize(std::vector<std::string> const pidList, std::string const &iapPayload)
{
    if (!IAPV4SubscriptionProductInfoView::init())
        return false;

    this->pidList.clear();
    this->pidList.assign(pidList.begin(), pidList.end());
    this->strIapPayload = iapPayload;

    setUseBackButton(true);
    initGroupPidList();

    addLabel("GROUP_1");
    for (auto item : pidList_group_1)
    {
        int index = getIndexFromVector(pidList, item);
        if (index != -1)
            addButton(item.c_str(), index);
    }

    addDottedLine();

    addLabel("GROUP_2");
    for (auto item : pidList_group_2)
    {
        int index = getIndexFromVector(pidList, item);
        if (index != -1)
            addButton(item.c_str(), index);
    }

    addDottedLine();

    addLabel("UNGROUPED");
    for(int i = 0; i < pidList.size(); i++) {
        if(getIndexFromVector(pidList_group_1, pidList[i]) == -1 && getIndexFromVector(pidList_group_2, pidList[i]) == -1) {
            addButton(pidList[i].c_str(), i);
        }
    }

    StartActivityIndicator();
    Postbox::posts([=](const Postbox::Posts *posts, Postbox::Error error) {
        if (posts == nullptr)
        {
            EndActivityIndicator();
            return;
        }

        Postbox::SubscriptionList subscriptionList = posts->subscription_list;
        if (subscriptionList.empty())
        {
            EndActivityIndicator();
            return;
        }

        for (auto subs : subscriptionList)
        {
            int index = getIndexFromVector(pidList, subs.product_id);
            if (index == -1)
                continue;

            cocos2d::ui::Button *button = (cocos2d::ui::Button *)BaseView::getUIWidget(index);
            button->setTitleColor(cocos2d::Color3B::RED);

            if (getIndexFromVector(pidList_group_1, subs.product_id) != -1)
            {
                group_1_item = subs.product_id;
            }
            else
            {
                if (getIndexFromVector(pidList_group_2, subs.product_id) != -1)
                {
                    group_2_item = subs.product_id;
                }
            }
        }
        EndActivityIndicator();
    });

    return true;
}

void IAPV4SubscriptionProductInfoView::initGroupPidList()
{
    pidList_group_1.clear();
    pidList_group_2.clear();

#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    pidList_group_1.push_back("sub01");
    pidList_group_1.push_back("sub10");
    pidList_group_1.push_back("sub11");
    pidList_group_1.push_back("sub30");

    pidList_group_2.push_back("sub013");
    pidList_group_2.push_back("sub103");
    pidList_group_2.push_back("sub303");
    pidList_group_2.push_back("sub113");
#else
    pidList_group_1.push_back("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.arshive003");
    pidList_group_1.push_back("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.arshive002");
    pidList_group_1.push_back("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.arshive001");

    pidList_group_2.push_back("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.arshive033");
    pidList_group_2.push_back("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.arshive032");
    pidList_group_2.push_back("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.arshive031");
#endif
}

void IAPV4SubscriptionProductInfoView::onButtonEvent(int buttonTag)
{
    std::string marketPid = pidList[buttonTag];
    std::string oldMarketPid = "";

    int index = getIndexFromVector(pidList_group_1, marketPid);
    if (index != -1)
    {
        oldMarketPid = group_1_item;
    }
    else
    {
        index = getIndexFromVector(pidList_group_2, marketPid);
        if (index != -1)
        {
            oldMarketPid = group_2_item;
        }
    }

    auto* param = new hive::IAPV4PurchaseParam();
    param->marketPid = marketPid;
    param->oldMarketPid = oldMarketPid;
    param->iapPayload = this->strIapPayload;

    IAPV4::purchaseSubscription(*param, [=](ResultAPI const &result, IAPV4Receipt const &receipt) {
//    IAPV4::purchaseSubscriptionUpdate(marketPid, oldMarketPid, "", [=](ResultAPI const &result, IAPV4Receipt const &receipt) {
        ResultUtils::logHead("IAPV4", "purchaseSubscription", result);
        if (ResultUtils::errorCheckResultAPI(result))
        {
            Logger::log(IAPV4Test::getReceiptToString(receipt).c_str());
            StartActivityIndicator();

            receiptVerify(receipt, Postbox::subscription, [=]() {
                EndActivityIndicator();
            });
        }
    });
}

void IAPV4SubscriptionProductInfoView::receiptVerify(hive::IAPV4Receipt receipt, Postbox::ItemType itemType, std::function<void()> callback)
{
    Postbox::verifyReceipt(receipt.bypassInfo, itemType, [=](const Postbox::VerifyReceipt *data, Postbox::Error error) mutable {
        if (data != nullptr)
        {
            //영수증 검증 성공, 피니쉬 처리.
            IAPV4::transactionFinish(receipt.product.marketPid, [=](ResultAPI const &result, std::string marketPid) mutable {

                if (callback)
                    callback();
            });
        }
        else
        {
            if (callback)
                callback();
        }
    });
}

int IAPV4SubscriptionProductInfoView::getIndexFromVector(std::vector<std::string> v, std::string data)
{
    auto itr = std::find(v.begin(), v.end(), data);
    if (itr != v.end())
    {
        return itr - v.begin();
    }
    return -1;
}
